package com.teamviewer.engage_plugin

import android.graphics.Color
import android.util.Log
import androidx.annotation.NonNull
import com.chatvisor.android.ChatvisorBase
import com.chatvisor.android.cobrowsing.screenObserver.IScreenGrabObserver
import com.chatvisor.android.user.ActiveUser
import com.chatvisor.android.liveview.result.ResultCallback
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** EngagePlugin */
class EngagePlugin: FlutterPlugin, MethodCallHandler {
  private lateinit var channel : MethodChannel
  private val screenGrabObserver: IScreenGrabObserver = FlutterScreenGrabObserver()

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "engage_plugin")
    channel.setMethodCallHandler(this)
      screenGrabObserver.updateRenderingEngine(flutterPluginBinding.textureRegistry)
  }

    private val resultCb : ResultCallback<String> = object:ResultCallback<String>{
        override fun onSuccess(result: com.chatvisor.android.liveview.result.Result<String>) {
            Log.i(result.tag, "SUCCESS: " + result.message + " " + result.result)
        }

        override fun onFailure(throwable: com.chatvisor.android.liveview.result.Result<Throwable>) {
            super.onFailure(throwable)
            Log.e( throwable.tag, "ERROR: ${throwable.message} ${throwable.result}")
        }
    }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
        "init" -> {
            val tenantId = call.argument<String>("tenantId")
            val token = call.argument<String>("token")
            val serverUrl = call.argument<String>("serverUrl")
            val cdnServerUrl = call.argument<String>("cdnServerUrl")
            val chatOverlayVisible = call.argument<Boolean>("chatOverlayVisible")
            val deeplinkUrls = call.argument<List<String>>("deeplinkUrls")
            val overlayBackgroundColor = call.argument<String>("overlayBackgroundColor")
            val controlsColor = call.argument<String>("controlsColor")
            val config = ChatvisorBase.CustomConfiguration(
                tenantId as String,
                token as String,
                serverUrl,
                cdnServerUrl,
                false,
                null,
                overlayBackgroundColor?.let { Color.parseColor(overlayBackgroundColor)},
                controlsColor?.let { Color.parseColor(controlsColor)},
            )
            ChatvisorBase.init(config)
            deeplinkUrls?.let { urls-> ChatvisorBase.liveView.setDeeplinkUrls(urls) }
            chatOverlayVisible?.let{ ChatvisorBase.liveView.setChatOverlayEnabled(it)}
            ChatvisorBase.liveView.setIsFlutter(true)
        }
        "openChat" -> {
            ChatvisorBase.webChat.open(resultCb)
        }
        "closeChat" -> {
            ChatvisorBase.webChat.close()
        }
        "startCoBrowsing" -> {
            ChatvisorBase.coBrowsing.startWithScreenGrabObserver(null, null, screenGrabObserver, resultCb)
        }
        "stopCoBrowsing" -> {
            ChatvisorBase.coBrowsing.stop()
        }
        "tagUser" -> {
            val email = call.argument<String>("email")
            val firstName = call.argument<String>("firstName")
            val lastName = call.argument<String>("lastName")
            val id = call.argument<String>("id")
            val labels = call.argument<ArrayList<String>>("labels")
            val assignedUser = call.argument<String>("assignedUser")
            val externalToken = call.argument<String>("externalToken")
            val additionalTaggingData = call.argument<Map<String,String>>("additionalTaggingData")
            val user = ActiveUser(id as String, email = email, firstname = firstName, lastname = lastName,
                labels = labels, assignedUser = assignedUser, externalToken = externalToken, additionalTaggingData = additionalTaggingData)
            ChatvisorBase.user.tag(user)
        }
        "clear" -> {
            ChatvisorBase.user.clear()
        }
        else -> {
          result.notImplemented()
        }
    }
  }

    private fun tagUser(id: String) {
        val labels: MutableList<String> = java.util.ArrayList()
        labels.add("foo")
        labels.add("bar")
        val user = ActiveUser(id, email = "john.doe@example.org", firstname = "John", lastname = id, labels = labels, assignedUser = "user@localhost")
        ChatvisorBase.user.tag(user)
    }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }
}
