import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:engage_plugin/engage_plugin.dart';
import 'package:flutter_dotenv/flutter_dotenv.dart';

Future main() async {
  await dotenv.load(fileName: ".env");
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  static var chatOverlayEnabled = false;
  final _engagePlugin = EngagePlugin();

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  var initConfig = InitConfig(
      dotenv.env['TENANT']!,
      dotenv.env['TOKEN']!,
      dotenv.env['SERVERURL']!,
      dotenv.env['CDNSERVERURL']!,
      chatOverlayEnabled,
      ["deeplinkurl(optional"],
      dotenv.env['OVERLAYBACKGROUNDCOLOR']!,
      dotenv.env['CONTROLCOLORS']!);

  var tagUserInfo =
      TagUserInfo(
          ["Labels", "list"],
          "Email",
          "FirstName",
          "LastName",
          "ID",
          "AssignedUser",
          "ExternalToken",
          {"Key":"Data"});

  Future<void> initPlatformState() async {
    try {
      await _engagePlugin.init(initConfig);
    } on PlatformException {
      'failed to init engage';
    }

    if (!mounted) {
      return;
    }

    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Engage plugin example'),
        ),
        body: Center(
            child: Column(
          children: [
            ElevatedButton(
                onPressed: (() => _engagePlugin.init(initConfig)),
                child: const Text('Init Plugin')),
            ElevatedButton(
                onPressed: () => _engagePlugin.openChat(),
                child: const Text('Open Chat')),
            ElevatedButton(
                onPressed: () => _engagePlugin.startCoBrowsing(),
                child: const Text('Open coBrowser')),
            ElevatedButton(
                onPressed: () => _engagePlugin.stopCoBrowsing(),
                child: const Text('Stop coBrowser')),
            ElevatedButton(
                onPressed: () => _engagePlugin.tagUser(tagUserInfo),
                child: const Text('Tag user')),
            ElevatedButton(
                onPressed: () => _engagePlugin.clear(),
                child: const Text('Clear')),
          ],
        )),
      ),
    );
  }
}
