import Flutter
import LiveView
import UIKit

public class SwiftEngagePlugin: NSObject, FlutterPlugin {
	let invalidArgsError = FlutterError( code: "invalidArgs", message: "Invalid argument(s)", details: nil)
	
	public static func register(with registrar: FlutterPluginRegistrar) {
		let channel = FlutterMethodChannel(name: "engage_plugin", binaryMessenger: registrar.messenger())
		let instance = SwiftEngagePlugin()
		registrar.addMethodCallDelegate(instance, channel: channel)
	}
	
	private func initPlugin(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
		guard let arguments = call.arguments as? NSDictionary,
			  let tenantId = arguments["tenantId"] as? String,
			  let token = arguments["token"] as? String else {
			result(invalidArgsError)
			return
		}
		
		let config = SetupConfiguration(tenantId: tenantId, token: token)
		
		if let serverUrl = arguments["serverUrl"] as? String {
			config.setServerUrl(serverUrl);
		}
		
		if let cdnServerUrl = arguments["cdnServerUrl"] as? String {
			config.setCdnUrl(cdnServerUrl)
		}

        if let overlayBackgroundColor = arguments["overlayBackgroundColor"] as? String {
             let bgColor = UIColor(hexString: overlayBackgroundColor)
                config.setOverlayBackgroundColor(bgColor)
        }

        if let controlsColor = arguments["controlsColor"] as? String {
             let cntrlColor = UIColor(hexString: controlsColor)
                config.setControlsColor(cntrlColor)
        }
		
		Chatvisor.setup(with: config)
        
        if let chatOverlayVisible = arguments["chatOverlayVisible"] as? Bool {
			Chatvisor.liveView.setChatOverlayEnabled(isVisible: chatOverlayVisible)
        }
        
		result(true)
	}
	
	private func tagUser(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
		guard let arguments = call.arguments as? NSDictionary,
			  let email = arguments["email"] as? String,
			  let firstName = arguments["firstName"] as? String,
			  let lastName = arguments["lastName"] as? String,
			  let id = arguments["id"] as? String,
              let externalToken = arguments["externalToken"] as? String,
              let additionalTaggingData = arguments["additionalTaggingData"] as? [String: String],
			  let labels = arguments["labels"] as? [String] else {
			result(invalidArgsError)
			return
		}
		
        let user = ActiveUser(id: id, firstname: firstName, lastname: lastName, email: email,tags: labels, externalToken: externalToken, additionalTaggingData: additionalTaggingData)
		Chatvisor.user.tag(user)
		result(true)
	}
	
	public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
		switch (call.method) {
		case "init":
			initPlugin(call, result: result)
			break
		case "openChat":
			Chatvisor.webChat.open();
			result(true)
			break
		case "closeChat":
            Chatvisor.webChat.close();
        	result(true)
        	break
		case "startCoBrowsing":
			Chatvisor.coBrowsing.start()
			result(true)
			break
		case "stopCoBrowsing":
			Chatvisor.coBrowsing.stop()
			result(true)
			break
		case "tagUser":
			tagUser(call, result: result)
			break
		case "clear":
			Chatvisor.user.clear()
			break
		default:
			result(FlutterMethodNotImplemented)
			break
		}
	}
}

extension UIColor {
    convenience init(hexString: String, alpha: CGFloat = 1.0) {
        let hexString: String = hexString.trimmingCharacters(in: CharacterSet.whitespacesAndNewlines)
        let scanner = Scanner(string: hexString)
        if (hexString.hasPrefix("#")) {
            scanner.scanLocation = 1
        }
        var color: UInt32 = 0
        scanner.scanHexInt32(&color)
        let mask = 0x000000FF
        let r = Int(color >> 16) & mask
        let g = Int(color >> 8) & mask
        let b = Int(color) & mask
        let red   = CGFloat(r) / 255.0
        let green = CGFloat(g) / 255.0
        let blue  = CGFloat(b) / 255.0
        self.init(red:red, green:green, blue:blue, alpha:alpha)
    }
    func toHexString() -> String {
        var r:CGFloat = 0
        var g:CGFloat = 0
        var b:CGFloat = 0
        var a:CGFloat = 0
        getRed(&r, green: &g, blue: &b, alpha: &a)
        let rgb:Int = (Int)(r*255)<<16 | (Int)(g*255)<<8 | (Int)(b*255)<<0
        return String(format:"#%06x", rgb)
    }
}
