import 'package:engage_plugin/engage_plugin.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'engage_plugin_platform_interface.dart';

/// An implementation of [EngagePluginPlatform] that uses method channels.
class MethodChannelEngagePlugin extends EngagePluginPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('engage_plugin');

  @override
  Future<void> init(InitConfig config) async {
    var dataToPass = <String, dynamic>{
      'tenantId': config.tenantId,
      'token': config.token,
      'serverUrl': config.serverUrl,
      'cdnServerUrl': config.cdnServerUrl,
      'chatOverlayVisible': config.chatOverlayVisible,
      'deeplinkUrls': config.deeplinkUrls,
      'overlayBackgroundColor': config.overlayBackgroundColor,
      'controlsColor': config.controlsColor
    };
    try {
      await methodChannel.invokeMethod<void>('init', dataToPass);
    } on PlatformException {
      'Failed to initialize.';
    }
  }

  @override
  Future<void> openChat() async {
    try {
      await methodChannel.invokeMethod<void>('openChat');
    } on PlatformException {
      'Failed to get open chat.';
    }
  }

  @override
  Future<void> closeChat() async {
    try {
      await methodChannel.invokeMethod<void>('closeChat');
    } on PlatformException {
      'Failed to get close chat.';
    }
  }

  @override
  Future<void> startCoBrowsing() async {
    try {
      await methodChannel.invokeMethod<void>('startCoBrowsing');
    } on PlatformException {
      'Failed to start coBrowsing.';
    }
  }

  @override
  Future<void> stopCoBrowsing() async {
    try {
      await methodChannel.invokeMethod<void>('stopCoBrowsing');
    } on PlatformException {
      'Failed to stop coBrowsing.';
    }
  }

  @override
  Future<void> tagUser(TagUserInfo info) async {
    var dataToPass = <String, dynamic>{
      'labels': info.labels,
      'email': info.email,
      'firstName': info.firstName,
      'lastName': info.lastName,
      'id': info.id,
      'assignedUser': info.assignedUser,
      'externalToken': info.externalToken,
      'additionalTaggingData': info.additionalTaggingData
    };
    try {
      await methodChannel.invokeMethod<void>('tagUser', dataToPass);
    } on PlatformException {
      'Failed to tag user.';
    }
  }

  @override
  Future<void> clear() async {
    try {
      await methodChannel.invokeMethod<void>('clear');
    } on PlatformException {
      'Failed to clear.';
    }
  }
}
